<?php

use Drupal\Component\Utility\Html;
use Drupal\contact\ContactFormInterface;
use Drupal\contact\Entity\Message;
use Drupal\contact\MessageForm;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\HtmlCommand;
use Drupal\Core\Ajax\PrependCommand;
use Drupal\Core\Ajax\ReplaceCommand;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Entity\EntityFormInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Render\Element\Checkboxes;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\Routing\Route;

/**
 * Implements hook_form_BASE_FORM_ID_alter(): contact_form_form.
 */
function improvements_form_contact_form_form_alter(array &$form, FormStateInterface $form_state) {
  $form_object = $form_state->getFormObject(); /** @var EntityFormInterface $form_object */
  $contact_form_entity = $form_object->getEntity(); /** @var ContactFormInterface $contact_form_entity */

  $form['recipients_new'] = [
    '#type' => 'textfield',
    '#title' => $form['recipients']['#title'],
    '#description' => $form['recipients']['#description'],
    '#default_value' => $form['recipients']['#default_value'],
    '#required' => TRUE,
    '#weight' => 0.001,
  ];
  $form['recipients']['#default_value'] = 'dummy@dummy.dummy';
  $form['recipients']['#required'] = FALSE;
  $form['recipients']['#access'] = FALSE;

  $form['additional_settings'] = [
    '#type' => 'details',
    '#title' => t('Additional settings'),
  ];

  $form['additional_settings']['page_title'] = [
    '#type' => 'textfield',
    '#title' => t('Page title'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'page_title'),
  ];

  $form['additional_settings']['name_label'] = [
    '#type' => 'textfield',
    '#title' => t('Name field label'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'name_label'),
  ];

  $form['additional_settings']['email_label'] = [
    '#type' => 'textfield',
    '#title' => t('E-mail field label'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'email_label'),
  ];

  $form['additional_settings']['subject_label'] = [
    '#type' => 'textfield',
    '#title' => t('Subject field label'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'subject_label'),
  ];

  $form['additional_settings']['message_label'] = [
    '#type' => 'textfield',
    '#title' => t('Message field label'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'message_label'),
  ];

  $form['additional_settings']['submit_button_text'] = [
    '#type' => 'textfield',
    '#title' => t('Submit button text'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'submit_button_text'),
  ];

  $form['additional_settings']['email_subject'] = [
    '#type' => 'textfield',
    '#title' => t('E-mail subject'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'email_subject'),
  ];

  $form['additional_settings']['required_fields'] = [
    '#type' => 'checkboxes',
    '#title' => t('Required fields'),
    '#options' => [
      'name' => t('Name'),
      'mail' => t('E-mail'),
      'subject' => t('Subject'),
      'message' => t('Message'),
    ],
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'required_fields', ['name', 'mail', 'subject', 'message']),
  ];

  $form['additional_settings']['ajax_submit'] = [
    '#type' => 'checkbox',
    '#title' => t('AJAX submit'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'ajax_submit'),
  ];

  $form['additional_settings']['hide_preview_button'] = [
    '#type' => 'checkbox',
    '#title' => t('Hide preview button'),
    '#default_value' => $contact_form_entity->getThirdPartySetting('improvements', 'hide_preview_button'),
  ];

  $form['#entity_builders'][] = 'improvements_contact_form_edit_form_entity_builder';
  $form['actions']['submit']['#submit'][] = 'improvements_contact_form_form_submit';
}

/**
 * Contact form entity builder.
 *
 * @see improvements_form_contact_form_edit_form_alter().
 */
function improvements_contact_form_edit_form_entity_builder(string $entity_type, ContactFormInterface $contact_form, array &$form, FormStateInterface $form_state) {
  // Save recipients
  $recipients = array_map('trim', explode(',', $form_state->getValue('recipients_new')));
  $contact_form->setRecipients($recipients);

  // Save additional settings
  $simple_settings = [
    'page_title',
    'subject_label',
    'name_label',
    'email_label',
    'message_label',
    'submit_button_text',
    'email_subject',
    'ajax_submit',
    'hide_preview_button',
  ];
  foreach ($simple_settings as $setting_name) {
    $contact_form->setThirdPartySetting('improvements', $setting_name, $form_state->getValue($setting_name));
  }

  $contact_form->setThirdPartySetting('improvements', 'required_fields', Checkboxes::getCheckedCheckboxes($form_state->getValue('required_fields')));
}

/**
 * Custom submit callback for contact edit form.
 *
 * @see improvements_form_contact_form_edit_form_alter().
 */
function improvements_contact_form_form_submit(array $form, FormStateInterface $form_state) {
  $form_object = $form_state->getFormObject(); /** @var EntityFormInterface $form_object */
  $contact_form_entity = $form_object->getEntity(); /** @var ContactFormInterface $contact_form_entity */
  $contact_form_entity_id = $contact_form_entity->id();
  $entity_field_manager = \Drupal::service('entity_field.manager'); /** @var EntityFieldManagerInterface $entity_field_manager */
  $contact_message_fields = $entity_field_manager->getFieldDefinitions('contact_message', $contact_form_entity_id);

  // Set new labels
  foreach (['name', 'mail', 'subject', 'message'] as $field_name) {
    if ($new_label = $form_state->getValue($field_name . '_label')) {
      $contact_message_fields[$field_name]->getConfig($contact_form_entity_id)->setLabel($new_label)->save();
    }
  }

  // Set required fields
  $required_fields = Checkboxes::getCheckedCheckboxes($form_state->getValue('required_fields'));
  foreach (['subject', 'message'] as $field_name) {
    $required = in_array($field_name, $required_fields);
    $contact_message_fields[$field_name]
      ->getConfig($contact_form_entity_id)
      ->setRequired($required)
      ->save();
  }
}

/**
 * Implements hook_form_BASE_FORM_ID_alter(): contact_message_form.
 *
 * @see \Drupal\contact\MessageForm::form()
 */
function improvements_form_contact_message_form_alter(array &$form, FormStateInterface $form_state, string $form_id) {
  $form_object = $form_state->getFormObject(); /** @var MessageForm $form_object */
  $contact_message_entity = $form_object->getEntity(); /** @var Message $contact_message_entity */
  $contact_form_entity = $contact_message_entity->getContactForm(); /** @var ContactFormInterface $contact_form_entity */
  $required_fields = $contact_form_entity->getThirdPartySetting('improvements', 'required_fields', []);

  // Change labels
  if ($name_label = $contact_form_entity->getThirdPartySetting('improvements', 'name_label')) {
    $form['name']['#title'] = $name_label;
  }
  if ($email_label = $contact_form_entity->getThirdPartySetting('improvements', 'email_label')) {
    $form['mail']['#title'] = $email_label;
  }

  // Restore "name" and "mail" elements for logged users
  if (\Drupal::currentUser()->isAuthenticated()) {
    if (isset($form['name'])) {
      $form['name']['#type'] = 'textfield';
      $form['name']['#default_value'] = $form['name']['#value'];
      unset($form['name']['#value']);
    }
    if (isset($form['mail'])) {
      $form['mail']['#type'] = 'email';
      $form['mail']['#default_value'] = $form['mail']['#value'];
      unset($form['mail']['#value']);
    }
  }

  // Change #required option
  if (isset($form['name'])) {
    $form['name']['#required'] = in_array('name', $required_fields);
  }
  if (isset($form['mail'])) {
    $form['mail']['#required'] = in_array('mail', $required_fields);
  }

  // Change submit button text
  if ($submit_button_text = $contact_form_entity->getThirdPartySetting('improvements', 'submit_button_text')) {
    $form['actions']['submit']['#value'] = $submit_button_text;
  }

  // Hide "preview" button
  if ($contact_form_entity->getThirdPartySetting('improvements', 'hide_preview_button')) {
    $form['actions']['preview']['#access'] = FALSE;
  }

  // AJAX submit
  if ($contact_form_entity->getThirdPartySetting('improvements', 'ajax_submit')) {
    $form_html_id = Html::getId($form_object->getFormId());
    $form['#id'] = $form_html_id;

    $form['actions']['submit']['#ajax'] = [
      'callback' => 'improvements_form_contact_message_form_ajax',
      'event' => 'click',
    ];
  }
}

/**
 * Contact form ajax submit callback.
 *
 * @see improvements_form_contact_message_form_alter()
 */
function improvements_form_contact_message_form_ajax(array &$form, FormStateInterface $form_state) {
  $response = new AjaxResponse();
  $form_selector = '#' . $form['#id'];

  if ($form_state->hasAnyErrors()) {
    // Update contact form
    $response->addCommand(new ReplaceCommand($form_selector, $form));
    // Add status messages in contact form
    $response->addCommand(new PrependCommand($form_selector, ['#type' => 'status_messages']));
  }
  else {
    // Delete all status messages
    \Drupal::messenger()->deleteAll();

    // Show success message instead form
    $contact_form_object = $form_state->getFormObject(); /** @var EntityFormInterface $contact_form_object */
    $contact_message_entity = $contact_form_object->getEntity(); /** @var Message $contact_message_entity */
    $contact_form_entity = $contact_message_entity->getContactForm(); /** @var ContactFormInterface $contact_form_entity */
    $complete_message = $contact_form_entity->getMessage() ?: t('Form is submitted');
    $response->addCommand(new HtmlCommand($form_selector, '<div class="contact-form__success-message">' . $complete_message . '</div>'));
  }

  return $response;
}

/**
 * Implements hook_page_title_alter().
 */
function improvements_page_title_alter(&$title, Request $request, Route $route) {
  if ($request->attributes->get('_route') == 'entity.contact_form.canonical') {
    $contact_form_entity = $request->attributes->get('contact_form'); /** @var ContactFormInterface $contact_form_entity */
    if ($contact_form_page_title = $contact_form_entity->getThirdPartySetting('improvements', 'page_title')) {
      $title = $contact_form_page_title;
    }
  }
}

/**
 * Implements hook_mail_ID_alter(): contact_page_mail.
 */
function improvements_mail_contact_page_mail_alter(array &$message) {
  $contact_form_entity = $message['params']['contact_form']; /** @var ContactFormInterface $contact_form_entity */

  // Change contact message email subject
  if ($contact_form_email_subject = $contact_form_entity->getThirdPartySetting('improvements', 'email_subject')) {
    $message['subject'] = $contact_form_email_subject;
  }
}
